Artifact = "maltego.STIX2.artifact"
AttackPattern = "maltego.STIX2.attack-pattern"
AutonomousSystem = "maltego.STIX2.autonomous-system"
Campaign = "maltego.STIX2.campaign"
CourseOfAction = "maltego.STIX2.course-of-action"
Directory = "maltego.STIX2.directory"
DomainName = "maltego.STIX2.domain-name"
EmailAddr = "maltego.STIX2.email-addr"
EmailMessage = "maltego.STIX2.email-message"
File = "maltego.STIX2.file"
Grouping = "maltego.STIX2.grouping"
Identity = "maltego.STIX2.identity"
Indicator = "maltego.STIX2.indicator"
Infrastructure = "maltego.STIX2.infrastructure"
IntrusionSet = "maltego.STIX2.intrusion-set"
Ipv4Addr = "maltego.STIX2.ipv4-addr"
Ipv6Addr = "maltego.STIX2.ipv6-addr"
Location = "maltego.STIX2.location"
MacAddr = "maltego.STIX2.mac-addr"
Malware = "maltego.STIX2.malware"
MalwareAnalysis = "maltego.STIX2.malware-analysis"
Mutex = "maltego.STIX2.mutex"
NetworkTraffic = "maltego.STIX2.network-traffic"
Note = "maltego.STIX2.note"
ObservedData = "maltego.STIX2.observed-data"
Opinion = "maltego.STIX2.opinion"
Process = "maltego.STIX2.process"
Relationship = "maltego.STIX2.relationship"
Report = "maltego.STIX2.report"
Sighting = "maltego.STIX2.sighting"
Software = "maltego.STIX2.software"
ThreatActor = "maltego.STIX2.threat-actor"
Tool = "maltego.STIX2.tool"
Url = "maltego.STIX2.url"
UserAccount = "maltego.STIX2.user-account"
Vulnerability = "maltego.STIX2.vulnerability"
WindowsRegistryKey = "maltego.STIX2.windows-registry-key"
XOpenctiCryptocurrencyWallet = "maltego.STIX2.x-opencti-cryptocurrency-wallet"
XOpenctiCryptographicKey = "maltego.STIX2.x-opencti-cryptographic-key"
XOpenctiHostname = "maltego.STIX2.x-opencti-hostname"
XOpenctiIncident = "maltego.STIX2.x-opencti-incident"
XOpenctiText = "maltego.STIX2.x-opencti-text"
XOpenctiUserAgent = "maltego.STIX2.x-opencti-user-agent"
X509Certificate = "maltego.STIX2.x509-certificate"
